package net.newtrack.sunmi_uhf.sunmi_uhf;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.os.RemoteException;

import androidx.annotation.NonNull;
import android.view.KeyEvent;

import io.flutter.Log;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.EventChannel;

import com.sunmi.rfid.*;
import com.sunmi.rfid.entity.DataParameter;

public class SunmiUhfPlugin implements FlutterPlugin, MethodCallHandler ,EventChannel.StreamHandler{
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;
  private EventChannel eventChannel;
  static byte btRepeat = 1;
  static public EventChannel.EventSink eventSink;
  private Handler uiThreadHandler = new Handler(Looper.getMainLooper());
  private Context context;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    context = flutterPluginBinding.getApplicationContext();
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "sunmi_uhf");
    channel.setMethodCallHandler(this);
    eventChannel = new EventChannel(flutterPluginBinding.getBinaryMessenger(), "sunmi_uhf.event");
    eventChannel.setStreamHandler(this);
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    switch (call.method) {
      case "init":
        init();
        result.success(RFIDManager.getInstance().isConnect());
        break;
      case "dispose":
        dispose();
        break;
      case "scan":
        SunmiUhfPlugin.scan();
        break;
      case "is_connect":
        result.success(RFIDManager.getInstance().isConnect());
        break;
      default:
        result.notImplemented();
        break;
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    RFIDManager.getInstance().disconnect();
    channel.setMethodCallHandler(null);
    channel = null;
    eventChannel.setStreamHandler(null);
    eventChannel = null;
  }

  private void init(){
    RFIDManager.getInstance().connect(context);
  }

  private void dispose(){
    RFIDManager.getInstance().getHelper().unregisterReaderCall();
  }

  static void scan(){
    RFIDManager.getInstance().getHelper().realTimeInventory(btRepeat);
  }

  private final ReaderCall mReaderCall = new ReaderCall() {
    @Override
    public void onSuccess(byte cmd, DataParameter params) throws RemoteException {
      uiThreadHandler.post(() -> {
        eventSink.success(null);
      });
    }

    @Override
    public void onTag(byte cmd, byte state, DataParameter tag) throws RemoteException {
      uiThreadHandler.post(() -> {
        eventSink.success(tag.getString("tag_epc"));
      });
  }

    @Override
    public void onFailed(byte cmd, byte errorCode, String msg) throws RemoteException {
      Log.e("scan error: ",msg);
    }
  };

  @Override
  public void onListen(Object arguments, EventChannel.EventSink events) {
    eventSink = events;
    RFIDManager.getInstance().getHelper().registerReaderCall(mReaderCall);
  }

  @Override
  public void onCancel(Object arguments) {
    eventSink = null;
    dispose();
  }
}
